;----------------------------------------------------------------------
;                                                                     
;                     Software License Agreement                      
;                                                                     
; The software supplied herewith by Microchip Technology Incorporated 
; (the "Company") for its PICmicro Microcontroller is intended and   
; supplied to you, the Companys customer, for use solely and         
; exclusively on Microchip PICmicro Microcontroller products.         
;                                                                     
; The software is owned by the Company and/or its supplier, and is     
; protected under applicable copyright laws. All rights are reserved.  
; Any use in violation of the foregoing restrictions may subject the  
; user to criminal sanctions under applicable laws, as well as to     
; civil liability for the breach of the terms and conditions of this  
; license.                                                             
;                                                                      
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,   
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR          
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.                    
;                                                                     
;----------------------------------------------------------------------
;
;   Filename:           xmit.asm
;   Date:               April 25, 2003
;   File Version:       1.0
;   Assembled using:    MPASM 03.30 Released
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;
;----------------------------------------------------------------------
;
;   Files required:
;                       p12f675.inc
;
;
;----------------------------------------------------------------------
;
;   Program Description 
;
;   This program demonstrates simple command & control and analog 
;   application demonstrations. To see each demonstration the user must 
;   load the appropriate receiver code examples:
;
;   rcvr_demo.asm
;
;   When a push button on the transmitter module is depressed, the 
;   corresponding LED is lit on the PICkit(tm) 1 FLASH Starter Kit.
;   Pressing transmitter module push button GP3 lights LED D0 on the
;   PICkit(tm) 1. Pressing push button GP4 lights LED D1.
;
;   rcvr_analog_display.asm
;
;   Pressing transmitter module push button GP3 lights LEDs D0-D7 on
;   the PICkit(tm) 1 with the upper 8-bit value read from the 
;   transmitter module 10-bit A/D channel 0 connected to potentiometer
;   GP0.  Pressing push button GP4 lights LEDs D0-D7 with the upper
;   8-bit value read from A/D channel 1 connected to potentiometer GP1.
;
;   The protocol is a simplified KeeLoq(r) protocol compatible with
;   the HCS101 fixed code products.  This receive code was adapted from 
;   Microchip Technology application note AN740.
;
;   The 10-bit analog value is placed into the 16-bit counter field of
;   the simplified KeeLoq(r) protocol.
;
;----------------------------------------------------------------------

    list        p=12f675        ; list directive to define processor
    #include    <p12f675.inc>   ; processor specific variable definitions

    errorlevel  -302            ; suppress message 302 from list file

; ---------------------------------------------------------------------
; Configuration Bits (Section 9.1 Configuration Bits)
; ---------------------------------------------------------------------
;
; Data Memory Code Protection bit:
; _CPD_ON = Enabled
; _CPD_OFF = Disabled
;
; Program Memory Code protection:
; _CP_ON = Enabled
; _CP_OFF = : Disabled
;
; Brown-out Detection Enable bit:
; _BODEN_ON = Enabled
; _BODEN_OFF = Disabled
;
; GP3/MCLR pin function select:
; _MCLRE_ON = GP3/MCLR pin function is /MCLR
; _MCLRE_OFF = GP3/MCLR pin function is digital I/O, 
;              /MCLR internally tied to Vdd
;
; Power-up Timer Enable bit:
; _PWRTE_ON = Enabled
; _PWRTE_OFF = Disabled
;
; Watchdog Timer Enable bit:
; _WDT_ON = Enabled
; _WDT_OFF = Disabled
;
; Oscillator Selction bits:
; _EXTRC_OSC_NOCLKOUT = CLKOUT function on GP4 pin, RC on GP5 pin.
; _EXTRC_OSC_CLKOUT = I/O function on GP4 pin, RC on GP5 pin.
; _INTRC_OSC_CLKOUT =  Internal oscillator, CLKOUT function on GP4 pin,
;                      I/O function on GP5 pin.
; _INTRC_OSC_NOCLKOUT = Internal oscillator, I/O function on GP4 and GP5 pins.
; _EC_OSC = I/O function on GP4 pin, CLKIN on GP5 pin.
; _HS_OSC = High speed crystal/resonator on GP4 and GP5 pins.
; _XT_OSC = Crystal/resonator on GP4 and GP5 pins.
; _LP_OSC = Low power crystal on GP4 and GP5 pins.
;
;
; ---------------------------------------------------------------------

    __CONFIG    _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT

;----------------------------------------------------------------------
; Variables  (Section 2.2 Data Memory Organization)
;----------------------------------------------------------------------

    ; Data Memory Organization (Section 2.2)
    ;
    ; Register locations 0x20 to 0x5F (64 bytes) are General Purpose 
    ; registers, implemented as static RAM and are mapped across both
    ; banks. 

    cblock  0x20

        w_temp                      ; used for context saving 
        status_temp                 ; used for context saving

        TEMP                        ; General Purpose Temporary register

        CSR0                        ; TX buffer shift register
        CSR1
        CSR2
        CSR3
        CSR4
        CSR5
        CSR6
        CSR7
        CSR8

        Count
        Count2
        BitCount
        TimeHi
        TimeLo

        FuncBits                    ; Function Bits

    endc

;----------------------------------------------------------------------
; Defines
;----------------------------------------------------------------------

    ; Set up GPIO Port (Section 3.0)
    ; Function of GPIO pins depend on:
    ;   Configuration Bits (CONFIG) (Section 9.1)
    ;   Weak Pull-up Register (WPU) (Section 3.2.1)
    ;   Interrupt-on-change GPIO Register (IOCB) (Section 3.2.2)
    ;   Option Register (OPTION_REG) (Register 4-1)
    ;   TIMER1 Control Register (T1CON) (Register 5-1)
    ;   Comparator Control Register (CMCON) (Section 6.0)
    ;   A/D Control Register (ADCON0) (Section 7.0) (PIC12F675 Only)

#define POT0    GPIO, 0             ; (Analog Input) Potentiometer GP0
#define POT1    GPIO, 1             ; (Analog Input) Potentiometer GP1
#define TXD     GPIO, 2             ; (Output) Transmit Data
#define PB3     GPIO, 3             ; (Input Only) Push button switch GP3
#define PB4     GPIO, 4             ; (Input) Push button switch GP4
#define RFENA   GPIO, 5             ; (Output) RF Enable

    ; Define for TRISIO Register (Section 3.1)
    ;
    ; GPIO is an 6-bit wide, bi-directional port. The corresponding data
    ; direction register is TRISIO. Setting a TRISIO bit (= 1) will make
    ; the corresponding GPIO pin and input. Clearing a TRISIO bit (= 0)
    ; will make the corresponding GPIO pin an output. The exception is
    ; GP3, which is input only and its TRIS bit will always read as a '1'.

    ; GPIO Pins = xx543210
#define GPTRIS  B'00011011'

    ; delays/timings

#define TGUARD  D'46'               ; 46 X TE
#define PREAMB  D'16'               ; Preamble length = 16 pulses


;----------------------------------------------------------------------
; Program Memory
;----------------------------------------------------------------------

    ; Program Memory Organization (Section 2.1)

        ORG     0x000               ; RESET Vector

        nop                         ; for ICD use
        goto    INITALIZE           ; goto INITALIZE


        ORG     0x004               ; Interrupt Vector

        movwf   w_temp              ; save W register
        swapf   STATUS, W           ; swap status to be saved into W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movwf   status_temp         ; save STATUS register

;----------------------------------------
; Interrupt Service Routine (ISR)
;
; Description: 
;   
;----------------------------------------

; Interrupt-on-change (Section 3.2.2 and 9.4.3)
;
; An input change on GPIO change sets the GPIF bit. The interrupt can
; be enabled/disabled be setting/clearing the GPIE bit. Individual pins
; are configured through the IOC register (see INITIALIZATION below).
;
; Clear the IOC interrupt by:
;  a) Any read or write of GPIO. This will end the mismatch condition.
;  b) Clear the flag bit GPIF

        movfw   GPIO                ; read GPIO
        bcf     INTCON, GPIF


;----------------------------------------

        swapf   status_temp, W      ; swap status_temp into W, sets bank to original state
        movwf   STATUS              ; restore STATUS register
        swapf   w_temp, F
        swapf   w_temp, W           ; restore W register

        retfie


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_READ
;
; Description: To read an EEPROM data memory location, the address is
;   written to the EEADR register and set control bit RD (EECON1<0>) to
;   initiate a read. Data is available in the EEDATA register the next
;   clock cycle.
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: W contains EEPROM address to be read
;   
; Output: W contains EEPROM data
;   
;----------------------------------------------------------------------

DATA_EEPROM_READ

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movwf   EEADR               ; move EEPROM address in W to EEADR
        bsf     EECON1, RD          ; initiate EEPROM read
        movf    EEDATA, W           ; move data to W

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_WRITE
;
; Description: To write an EEPROM data memory location, the address is
;   written to the EEADR register, data to the EEDATA register, then
;   execute a required sequence of instructions.
;   
; CAUTION: Interrupts are disable during this subroutine
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: Address = EEADR, Data = EEDATA
;   
; Output: none
;   
;----------------------------------------------------------------------

DATA_EEPROM_WRITE

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bsf     EECON1, WREN        ; EEPROM Write Enable: allow write cycles
        bcf     INTCON, GIE         ; disable global interrupts
                                    ; *** required sequence, do not alter ***
        movlw   0x55
        movwf   EECON2
        movlw   0xAA
        movwf   EECON2
        bsf     EECON1, WR          ; initiate EEPROM write
                                    ; *** end required sequence ***
        bsf     INTCON, GIE         ; enable global interrupts
        bcf     EECON1, WREN        ; EEPROM Write Enable: inhibit write cycles

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return


;----------------------------------------------------------------------
; Subroutine READ_ANALOG_AN0; READ_ANALOG_AN1
;
; Description: Read analog channel 0 (AN0) or 1 (AN1).
;   
; Constants: none
;   
; Global Variables: none
;   
; Initialization: none
;   
; Output: ADRESH and ADRESL contain 10-bit A/D result justified 
;   according to ADCON0, ADFM bit.
;   
;----------------------------------------------------------------------

READ_ANALOG_AN0

        bcf     ADCON0, CHS1        ; select analog channel AN0
        bcf     ADCON0, CHS0

        goto    READ_ANALOG

READ_ANALOG_AN1

        bcf     ADCON0, CHS1        ; select analog channel AN1
        bsf     ADCON0, CHS0

READ_ANALOG

        bsf     ADCON0, ADON        ; Turn on ADC module

        ; After selecting a new channel, allow for sufficent sample time.
        ; The amount of sample time depends on the charging time of the
        ; internal charge holding capacitor (Section 7.2).

        movlw   D'6'                ; At 4 MHz, a 22us delay
        movwf   TEMP                ; (22us = 2us + 6 * 3us + 1us)
        decfsz  TEMP, F
        goto    $-1

        bsf     ADCON0, GO          ; start A/D conversion

        btfsc   ADCON0, GO          ; has A/D conversion completed?
        goto    $-1

        bcf     ADCON0, ADON        ; Turn off ADC module (consumes no operating current)

        return


;----------------------------------------------------------------------
; Subroutine: WaitxTE
;   
; Description: 
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: W x 400us length of delay required
;   
; Output: Count, Count2   as decrementing counters
;   
;----------------------------------------------------------------------

WaitxTE
        movwf   Count2      ; [1]

waitxlp            
        movlw   D'79'       ; [1]
        movwf   Count       ; [1]

wait400lp
        nop                 ; [1]
        nop                 ; [1]
        decfsz  Count,F     ; [1]
         goto    wait400lp  ; [2]
;                          --------
;                         79 x 5 = 395us

        decfsz  Count2,F    ; [1]
         goto    waitxlp    ; [2]

        retlw   0           ; [2]

;                       total 2 (call) + W x (395 + 5) + 2 (return)
;                       w = 1 -> 406us @4MHz
;                       w = 2 -> 806us @4MHz


;----------------------------------------------------------------------
; Initialize PICmicro (PIC12F675)
;----------------------------------------------------------------------

INITALIZE

; Disable global interrupts during initialization

        bcf     INTCON, GIE         ; disable global interrupts


;----------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (Section 2.2.2.7)
;
; A calibration instruction is programmed into the last location of
; program memory. This instruction is a RETLW XX, where the literal is
; the calibration value. The literal is placed in the OSCCAL register
; to set the calibration of the internal oscillator.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        call    0x3FF               ; retrieve factory calibration value
        movwf   OSCCAL              ; update register with factory cal value

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; GPIO Port (Section 3.0)
;
; Store GPTRIS value defined above into the TRISIO direction register

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movlw   GPTRIS
        movwf   TRISIO              ; Write to TRISIO register

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Comparator Module (Section 6.0)
;
; The PIC12F629/675 devices have one analog comparator. The inputs to
; the comparator are multiplexed with the GP0 and GP1 pins. There is 
; an on-chip Comparator Voltage Reference that can also be applied to
; an input of the comparator. In addition, GP2 can be configured as
; the comparator output. The Comparator Control Register (CMCON)
; contains bits to control the comparator. The Voltage Reference
; Control Register (VRCON) controls the voltage reference module.


        ; Comparator Configuration (Figure 6-2)
;        bcf     CMCON, CINV         ; Comparator Output Inversion: not inverted
;        bcf     CMCON, COUT         ; Comparator Output bit: Vin+ < Vin-
;        bcf     CMCON, CIS          ; Comparator Input Switch: Vin- connectos to Cin-

        ; CM2:CM0 = 111 - Comparator Off (lowest power)
        bsf     CMCON, CM2          ; Comparator Mode bit 2
        bsf     CMCON, CM1          ; Comparator Mode bit 1
        bsf     CMCON, CM0          ; Comparator Mode bit 0

        ; VRCON (Register 6-2)
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     VRCON, VREN         ; CVref circuit: powered down, no Idd drain

;        bcf     VRCON, VRR          ; CVref Range Selection: High Range

;        bcf     VRCON, VR3          ; CVref value selection bit 3
;        bcf     VRCON, VR2          ; CVref value selection bit 2
;        bcf     VRCON, VR1          ; CVref value selection bit 1
;        bcf     VRCON, VR0          ; CVref value selection bit 0

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Analog-to-Digital Converter (A/D) Module (Section 7.0) (PIC12F675 Only)
;
; The analog-to-digital converter (A/D) allows conversion of an analog
; input signal to a 10-bit binary representation of that signal. The
; PIC12F675 has four analog inputs multiplexed into one sample and hold
; circuit. There are two registers to control the functions of the A/D
; module:
;   A/D Control Register (ADCON0)
;   Analog Select Register (ANSEL)
;
; Note: When using GPIO pins as analog inputs, ensure the TRISIO register
;       bits are set (= 1) for input.

        bcf     ADCON0, ADFM        ; A/D Result Formed: left justified
        bcf     ADCON0, VCFG        ; Voltage Reference: Vdd
        bcf     ADCON0, ADON        ; ADC is shut-off and consumes no operating current

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        ; select A/D Conversion Clock Source: Fosc/8
        bcf     ANSEL, ADCS2        ; A/D Conversion Clock Select bit 2
        bcf     ANSEL, ADCS1        ; A/D Conversion Clock Select bit 1
        bsf     ANSEL, ADCS0        ; A/D Conversion Clock Select bit 0

        ; select GPIO pins that will be analog inputs: GP0/AN0, GP1/AN1
        bcf     ANSEL, ANS3         ; Analog Select GP4/AN3: digital I/O
        bcf     ANSEL, ANS2         ; Analog Select GP2/AN2: digital I/O
        bsf     ANSEL, ANS1         ; Analog Select GP1/AN1: analog input
        bsf     ANSEL, ANS0         ; Analog Select GP0/AN0: analog input

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; TIMER1 Module with Gate Control (Section 5.0)
;
; The TIMER1 Control Register (T1CON) is used to enable/disable TIMER1
; and select various features of the TIMER1 module.

        bcf     T1CON, TMR1ON       ; TIMER1: stopped

        bcf     T1CON, TMR1CS       ; TIMER1 Clock Source Select: Internal Clock (Fosc/4)

        bcf     T1CON, NOT_T1SYNC   ; TIMER1 External Clock Input Sync Control: Syncronize external clock input

        ; T1OSCEN only if INTOSC without CLKOUT oscillator is active, else ignored
        bcf     T1CON, T1OSCEN      ; LP Oscillator Enable Control: LP oscillator off

        ; TIMER1 Input Prescale Select: 1:1
        bcf     T1CON, T1CKPS1      ; TIMER1 Input Clock Prescale Select bit 1
        bcf     T1CON, T1CKPS0      ; TIMER1 Input Clock Prescale Select bit 0

        ; TMR1GE only if TMR1ON = 1, else ignored
        bcf     T1CON, TMR1GE       ; TIMER1 Gate Enable: on


;----------------------------------------
; Weak Pull-up Register (WPU) (Section 3.2.1)
;
; Each of the GPIO pins, except GP3, has an individually configurable
; weak internal pull-up. Control bits WPUx enable or disable each 
; pull-up. Refer to Register 3-1. Each weak pull-up is automatically
; turned off when the port pin is configured as an output. The pull-ups
; are disabled on a Power-on Reset by the NOT_GPPU bit (see OPTION_REG below).

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

;     GPIO Pins = xx54x210
        movlw   B'00010000'         ; GP4 pull-up enabled
        movwf   WPU

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; OPTION Register (OPTION_REG) (Section 2.2.2.2)
; TIMER0 Module (Section 4.0)
;
; The OPTION_REG contains control bits to configure:
;   Weak pull-ups on GPIO (see also WPU Register above)
;   External GP2/INT interrupt
;   TMR0
;   TMR0/WDT prescaler

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     OPTION_REG, NOT_GPPU  ; GPIO pull-ups: enabled

        bsf     OPTION_REG, INTEDG  ; Interrupt Edge: on rising edge of GP2/INT pin

        bcf     OPTION_REG, T0CS    ; TMR0 Clock Source: internal instruction cycle (CLKOUT)
        bcf     OPTION_REG, T0SE    ; TMR0 Source Edge: increment low-to-high transition on GP2/T0CKI pin

        bcf     OPTION_REG, PSA     ; Prescaler Assignment: assigned to TIMER0

        ; TMR0 Prescaler Rate: 1:2
        bcf     OPTION_REG, PS2     ; Prescaler Rate Select bit 2
        bcf     OPTION_REG, PS1     ; Prescaler Rate Select bit 1
        bcf     OPTION_REG, PS0     ; Prescaler Rate Select bit 0

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Interrupt-on-Change Register (IOCB) (Section 3.2.2)
;
; Each of the GPIO pins is individually configurable as an interrupt-
; on-change pin. Control bits IOCBx enable or disable the interrupt 
; function for each pin. Refer to Register 3-2. The interrupt-on-change
; is disabled on a Power-on Reset.
;
; Note: Global interrupt enables (GIE and GPIE) must be enabled for
;       individual interrupts to be recognized.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

    ; GPIO Pins = xx543210
        movlw   b'00011000'
        movwf   IOCB                ; Interrupt-on-change enabled: GP3, GP4

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Peripheral Interrupt Enable Register (PIE1) (Section 2.2.2.4)
;
; The PIE1 register contains peripheral interrupt enable bits.
;
; Note: The PEIE bit (INTCON<6>) must be set to enable any 
;       peripheral interrupt.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     PIE1, EEIE          ; EE Write Complete Interrupt: disabled
        bcf     PIE1, ADIE          ; A/D Converter Interrupt (PIC12F675 Only): disabled
        bcf     PIE1, CMIE          ; Comparator Interrupt: disabled
        bcf     PIE1, TMR1IE        ; TMR1 Overflow Interrupt: disabled

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Interrupt Control Register (INTCON) (Section 2.2.2.3)
;
; The INTCON register contains enable and disable flag bits for TMR0
; register overflow, GPIO port change and external GP2/INT pin 
; interrupts.

        bcf     INTCON, PEIE        ; disable Peripheral Interrupt Enable bit
        bcf     INTCON, T0IE        ; disable TMR0 Overflow Interrupt Enable bit
        bcf     INTCON, INTE        ; disable GP2/INT External Interrupt Enable bit
        bsf     INTCON, GPIE        ; enable Port Change Interrupt Enable bit

;        bcf     INTCON, GIE         ; disable global interrupts


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Main Program
;----------------------------------------------------------------------
;----------------------------------------------------------------------

MAIN

        bcf     RFENA           ; Disable Transmitter
        bsf     INTCON, GIE     ; enable global interrupts

;----------------------------------
; Scan push buttons
;----------------------------------

SCANPB

        movlw   0x00            ; load zero into W
        movwf   FuncBits        ; clear Function Bits register

        btfsc   PB3             ; Push Button GP3 pressed?
         goto   SPB1
        movlw   0x23            ; Function S0 selected
        iorwf   FuncBits, F
        call    READ_ANALOG_AN0 ; read analog channel AN0

SPB1
        btfsc   PB4             ; Push Button GP4 depressed?
         goto   SPB2            ; no, jump over
        movlw   0x43            ; Function S1 selected
        iorwf   FuncBits, F
        call    READ_ANALOG_AN1 ; read analog channel AN1

SPB2
        movlw   0xFF
        andwf   FuncBits, W     ; Was any switch depressed?
        btfss   STATUS, Z
         goto    XMIT            ; Yes, transmit buffer

        bcf     RFENA           ; No, disable Transmitter
        sleep                   ; Put PICmicro to sleep
        goto    SCANPB          ; Upon wake-up on pin change, scan push buttons


;----------------------------------
; fill in transmission buffer
;----------------------------------

XMIT

        bsf     RFENA           ; Enable Transmitter

        movlw   0x73            ; send Serial number  
        movwf   CSR0

        movfw   FuncBits        ; send Function Bits
        movwf   CSR1

                                ; send 16-bit Counter
                                ;
                                ; send analog value (this is modified from the fixed 
                                ;   KeeLoq(r) protocol; these two fields are normally 
                                ;   used for 16-bit counter value)

        bsf     STATUS, RP0     ; ----- Select Bank 1 -----
        movfw   ADRESL          ; ADRESL Result
        bcf     STATUS, RP0     ; ----- Select Bank 0 -----
        movwf   CSR2

        movfw   ADRESH          ; ADRESH Result
        movwf   CSR3

        movlw   0x56            ; send 32-bit serial number 1 (ignored by receiver)
        movwf   CSR4

        movlw   0x34
        movwf   CSR5

        movlw   0x12
        movwf   CSR6

        movlw   0x20
        movwf   CSR7

        movlw   0x55            ; send Flags
        movwf   CSR8


;----------------------------------
; Transmission Loop
;----------------------------------

TXLoop

        ; send preamble (50% duty cycle)

Preamble
        movlw   PREAMB
        movwf   BitCount        ; init number of preamble bits

PreL
        bsf     TXD             ; ON
        movlw   1
        call    WaitxTE         ; delay
        bcf     TXD             ; OFF
        movlw   1
        call    WaitxTE         ; delay

        decfsz  BitCount,F      ; loop
        goto    PreL
            
; sync pause

TXloop      
        movlw   D'10'           ; Theader = 10 x Te
        call    WaitxTE
            
; send 72 bit pattern

        movlw   CSR0            ; lsb first 
        movwf   FSR
            
TXNextByte
        movlw   D'8'
        movwf   BitCount

TXNextBit
        rrf     INDF,W           ; 8 bit rotate
        rrf     INDF,F           ; Carry contain lsb
        BC      ONE
ZERO            
        movlw   2               ; 
        movwf   TimeHi          ;   +---+---+   +--
        movlw   1               ;   |       |   |
        movwf   TimeLo          ;---+       +---+
        goto    Trasm_BIT       ;   |  2Te   Te |

ONE
        movlw   1               ;              
        movwf   TimeHi          ;   +---+       +--
        movlw   2               ;   |   |       |
        movwf   TimeLo          ;---+   +---+---+
                                ;   | Te   2Te  |
Trasm_BIT
        bsf     TXD             ; ON
        movf    TimeHi,W          
        call    WaitxTE

        bcf     TXD             ; OFF
        movf    TimeLo,W          
        call    WaitxTE

        decfsz  BitCount,F
        goto    TXNextBit       ; loop on bits

        incf    FSR,F
        movlw   CSR8+1          ; check if finished
        xorwf   FSR,W
        andlw   0x1F
        BNZ     TXNextByte

; guard time
        movlw   TGUARD
        call    WaitxTE

        goto    SCANPB          ; go back and scan for push button presses


;----------------------------------------------------------------------
; Data EEPROM Memory (Section 8.0)
;
; PIC12F629/675 devices have 128 bytes of data EEPROM with address
; range 0x00 to 0x7F.

        ; Initialize Data EEPROM Memory locations

;        ORG 0x2100
;        DE  0x00, 0x01, 0x02, 0x03


;----------------------------------------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (OSCCAL) (Section 2.2.2.7)
;
; The below statements are placed here so that the program can be
; simulated with MPLAB SIM.  The programmer (PICkit or PROMATE II)
; will save the actual OSCCAL value in the device and restore it.
; The value below WILL NOT be programmed into the device.

        org     0x3ff
        retlw    0x80

;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
