;----------------------------------------------------------------------
;                                                                     
;                     Software License Agreement                      
;                                                                     
; The software supplied herewith by Microchip Technology Incorporated 
; (the "Company") for its PICmicro Microcontroller is intended and   
; supplied to you, the Companys customer, for use solely and         
; exclusively on Microchip PICmicro Microcontroller products.         
;                                                                     
; The software is owned by the Company and/or its supplier, and is     
; protected under applicable copyright laws. All rights are reserved.  
; Any use in violation of the foregoing restrictions may subject the  
; user to criminal sanctions under applicable laws, as well as to     
; civil liability for the breach of the terms and conditions of this  
; license.                                                             
;                                                                      
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,   
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR          
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.                    
;                                                                     
;----------------------------------------------------------------------
;
;   Filename:           xmittest.asm
;   Date:               May 8, 2003
;   File Version:       1.0
;   Assembled using:    MPASM 03.30 Released
;
;   Author:             Myron Loewen
;   Company:            Microchip Technology Inc.
;
;
;----------------------------------------------------------------------
;
;   Files required:
;                       p12f675.inc
;
;
;----------------------------------------------------------------------
;
;   Program Description 
;
;   This program implements two tests that can be used to verify the RF
;   performance of the rfPIC Development Kit transmitter modules using 
;   a spectrum anaylzer and calibrated antenna.
;
;   The tests start when the indicated push button is pressed and stop 
;   when the push button is released. The processor SLEEPs between tests
;   to conserve battery power.
;
;   Peak Power Measurement - Push button GP3 (SW2)
;
;   This test generates a constant unmodulated output for measuring peak
;   output power. This test canbe used for regulatory pre-compliance
;   testing antenna tuning.
;
;   Data Modulation and Bandwith Measurement - Push button GP4 (SW1)
;
;   Set potentiometer GP0 in one of four quarter positions to choose 
;   maximum, high, medium, or low speed signal rate. Then press GP3 (SW2) 
;   and fine adjust potentiometer GP1 for the desired signal rate. 
;   The signal is a square wave and does not exactly match the PWM or 
;   Manchester spectrums, but is quite useful to determine receiver 
;   performance and FSK deviation. Listed below is a table of approximate
;   signal rates. Note that increasing the analog voltage decreases the 
;   signal rate and that the NRZ bps is double the modulation frequency.
;
;   GP0         NRZ bps	(0-100% GP1)    NRZ bit width (0-100% GP1)
;   -------     --------------------    --------------------------
;   0-25%       52.6 - 3.64 kbps        19 - 274 usec
;   25-50%      3.64 - 1.88 kbps        275 - 530 usec
;   50-75%      1.88 - 1.27 kbps        531 - 786 usec
;   75-100%     1.27 - 0.96 kbps        787 - 1042 usec
;
;----------------------------------------------------------------------

    list        p=12f675        ; list directive to define processor
    #include    <p12f675.inc>   ; processor specific variable definitions

    errorlevel  -302            ; suppress message 302 from list file

; ---------------------------------------------------------------------
; Configuration Bits (Section 9.1 Configuration Bits)
; ---------------------------------------------------------------------
;
; Data Memory Code Protection bit:
; _CPD_ON = Enabled
; _CPD_OFF = Disabled
;
; Program Memory Code protection:
; _CP_ON = Enabled
; _CP_OFF = : Disabled
;
; Brown-out Detection Enable bit:
; _BODEN_ON = Enabled
; _BODEN_OFF = Disabled
;
; GP3/MCLR pin function select:
; _MCLRE_ON = GP3/MCLR pin function is /MCLR
; _MCLRE_OFF = GP3/MCLR pin function is digital I/O, 
;              /MCLR internally tied to Vdd
;
; Power-up Timer Enable bit:
; _PWRTE_ON = Enabled
; _PWRTE_OFF = Disabled
;
; Watchdog Timer Enable bit:
; _WDT_ON = Enabled
; _WDT_OFF = Disabled
;
; Oscillator Selction bits:
; _EXTRC_OSC_NOCLKOUT = CLKOUT function on GP4 pin, RC on GP5 pin.
; _EXTRC_OSC_CLKOUT = I/O function on GP4 pin, RC on GP5 pin.
; _INTRC_OSC_CLKOUT =  Internal oscillator, CLKOUT function on GP4 pin,
;                      I/O function on GP5 pin.
; _INTRC_OSC_NOCLKOUT = Internal oscillator, I/O function on GP4 and GP5 pins.
; _EC_OSC = I/O function on GP4 pin, CLKIN on GP5 pin.
; _HS_OSC = High speed crystal/resonator on GP4 and GP5 pins.
; _XT_OSC = Crystal/resonator on GP4 and GP5 pins.
; _LP_OSC = Low power crystal on GP4 and GP5 pins.
;
;
; ---------------------------------------------------------------------

    __CONFIG    _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT

;----------------------------------------------------------------------
; Variables  (Section 2.2 Data Memory Organization)
;----------------------------------------------------------------------

    ; Data Memory Organization (Section 2.2)
    ;
    ; Register locations 0x20 to 0x5F (64 bytes) are General Purpose 
    ; registers, implemented as static RAM and are mapped across both
    ; banks. 

    cblock  0x20

        w_temp                      ; used for context saving 
        status_temp                 ; used for context saving

        Count
        CountHi
        CountSave
    endc

;----------------------------------------------------------------------
; Defines
;----------------------------------------------------------------------

    ; Set up GPIO Port (Section 3.0)
    ; Function of GPIO pins depend on:
    ;   Configuration Bits (CONFIG) (Section 9.1)
    ;   Weak Pull-up Register (WPU) (Section 3.2.1)
    ;   Interrupt-on-change GPIO Register (IOCB) (Section 3.2.2)
    ;   Option Register (OPTION_REG) (Register 4-1)
    ;   TIMER1 Control Register (T1CON) (Register 5-1)
    ;   Comparator Control Register (CMCON) (Section 6.0)
    ;   A/D Control Register (ADCON0) (Section 7.0) (PIC12F675 Only)

#define POT0    GPIO, 0             ; (Analog Input) Potentiometer GP0
#define POT1    GPIO, 1             ; (Analog Input) Potentiometer GP1
#define TXD     GPIO, 2             ; (Output) Transmit Data
#define PB3     GPIO, 3             ; (Input Only) Push button switch GP3
#define PB4     GPIO, 4             ; (Input) Push button switch GP4
#define RFENA   GPIO, 5             ; (Output) RF Enable

    ; Define for TRISIO Register (Section 3.1)
    ;
    ; GPIO is an 6-bit wide, bi-directional port. The corresponding data
    ; direction register is TRISIO. Setting a TRISIO bit (= 1) will make
    ; the corresponding GPIO pin and input. Clearing a TRISIO bit (= 0)
    ; will make the corresponding GPIO pin an output. The exception is
    ; GP3, which is input only and its TRIS bit will always read as a '1'.

    ; GPIO Pins = xx543210
#define GPTRIS  B'00011011'

;----------------------------------------------------------------------
; Program Memory
;----------------------------------------------------------------------

    ; Program Memory Organization (Section 2.1)

        ORG     0x000               ; RESET Vector

        nop                         ; for ICD use
        goto    INITALIZE           ; goto INITALIZE


        ORG     0x004               ; Interrupt Vector

        movwf   w_temp              ; save W register
        swapf   STATUS, W           ; swap status to be saved into W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movwf   status_temp         ; save STATUS register

;----------------------------------------
; Interrupt Service Routine (ISR)
;
; Description: 
;   
;----------------------------------------

; Interrupt-on-change (Section 3.2.2 and 9.4.3)
;
; An input change on GPIO change sets the GPIF bit. The interrupt can
; be enabled/disabled be setting/clearing the GPIE bit. Individual pins
; are configured through the IOC register (see INITIALIZATION below).
;
; Clear the IOC interrupt by:
;  a) Any read or write of GPIO. This will end the mismatch condition.
;  b) Clear the flag bit GPIF

        movf    GPIO, W             ; read GPIO
        bcf     INTCON, GPIF


;----------------------------------------

        swapf   status_temp, W      ; swap status_temp into W, sets bank to original state
        movwf   STATUS              ; restore STATUS register
        swapf   w_temp, F
        swapf   w_temp, W           ; restore W register

        retfie

;----------------------------------------------------------------------
; Initialize PICmicro (PIC12F675)
;----------------------------------------------------------------------

INITALIZE

;----------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (Section 2.2.2.7)
;
; A calibration instruction is programmed into the last location of
; program memory. This instruction is a RETLW XX, where the literal is
; the calibration value. The literal is placed in the OSCCAL register
; to set the calibration of the internal oscillator.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        call    0x3FF               ; retrieve factory calibration value
        movwf   OSCCAL              ; update register with factory cal value

;----------------------------------------
; GPIO Port (Section 3.0)
;
; Store GPTRIS value defined above into the TRISIO direction register
        movlw   GPTRIS
        movwf   TRISIO              ; Write to TRISIO register

;----------------------------------------
; Weak Pull-up Register (WPU) (Section 3.2.1)
;
; Each of the GPIO pins, except GP3, has an individually configurable
; weak internal pull-up. Control bits WPUx enable or disable each 
; pull-up. Refer to Register 3-1. Each weak pull-up is automatically
; turned off when the port pin is configured as an output. The pull-ups
; are disabled on a Power-on Reset by the NOT_GPPU bit (see OPTION_REG below).

;     GPIO Pins = xx54x210
        movlw   B'00010000'         ; GP4 pull-up enabled
        movwf   WPU

;----------------------------------------
; Interrupt-on-Change Register (IOCB) (Section 3.2.2)
;
; Each of the GPIO pins is individually configurable as an interrupt-
; on-change pin. Control bits IOCBx enable or disable the interrupt 
; function for each pin. Refer to Register 3-2. The interrupt-on-change
; is disabled on a Power-on Reset.
;
; Note: Global interrupt enables (GIE and GPIE) must be enabled for
;       individual interrupts to be recognized.

    ; GPIO Pins = xx543210
        movlw   b'00011000'
        movwf   IOCB                ; Interrupt-on-change enabled: GP3, GP4

;----------------------------------------
; Peripheral Interrupt Enable Register (PIE1) (Section 2.2.2.4)
;
; The PIE1 register contains peripheral interrupt enable bits.
;
; Note: The PEIE bit (INTCON<6>) must be set to enable any 
;       peripheral interrupt.

        clrf    PIE1				; Peripheral Interrupts Disabled

;----------------------------------------
; Interrupt Control Register (INTCON) (Section 2.2.2.3)
;
; The INTCON register contains enable and disable flag bits for TMR0
; register overflow, GPIO port change and external GP2/INT pin 
; interrupts.

        movlw   b'10001000'			; Enable only port change interrupt
        movwf   INTCON

;----------------------------------------
; OPTION Register (OPTION_REG) (Section 2.2.2.2)
; TIMER0 Module (Section 4.0)
;
; The OPTION_REG contains control bits to configure:
;   Weak pull-ups on GPIO (see also WPU Register above)
;   External GP2/INT interrupt
;   TMR0
;   TMR0/WDT prescaler

        movlw   b'01111111'			; Enable GPIO pullups
        movwf   OPTION_REG

;----------------------------------------
; Analog-to-Digital Converter (A/D) Module (Section 7.0) (PIC12F675 Only)
;
; The analog-to-digital converter (A/D) allows conversion of an analog
; input signal to a 10-bit binary representation of that signal. The
; PIC12F675 has four analog inputs multiplexed into one sample and hold
; circuit. There are two registers to control the functions of the A/D
; module:
;   A/D Control Register (ADCON0)
;   Analog Select Register (ANSEL)
;
; Note: When using GPIO pins as analog inputs, ensure the TRISIO register
;       bits are set (= 1) for input.

        movlw   b'00010011'			; Enable AN0 and AN1 with A/D Clk = Fosc/8
        movwf   ANSEL

        ; VRCON (Register 6-2)
        clrf    VRCON				; CVref circuit: powered down, no Idd drain

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        clrf	ADCON0				; left justified, Vdd ref, A/D off

;----------------------------------------
; Comparator Module (Section 6.0)
;
; The PIC12F629/675 devices have one analog comparator. The inputs to
; the comparator are multiplexed with the GP0 and GP1 pins. There is 
; an on-chip Comparator Voltage Reference that can also be applied to
; an input of the comparator. In addition, GP2 can be configured as
; the comparator output. The Comparator Control Register (CMCON)
; contains bits to control the comparator. The Voltage Reference
; Control Register (VRCON) controls the voltage reference module.

        ; CM2:CM0 = 111 - Comparator Off (lowest power)
        movlw   b'00000111'			; Comparator inputs grounded
        movwf   CMCON

;----------------------------------------
; TIMER1 Module with Gate Control (Section 5.0)
;
; The TIMER1 Control Register (T1CON) is used to enable/disable TIMER1
; and select various features of the TIMER1 module.

        clrf    T1CON				; TIMER1 not used

;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Main Program
;----------------------------------------------------------------------
;----------------------------------------------------------------------

MAIN
        bsf     INTCON, GIE         ; enable global interrupts
        bcf     TXD
		
        bcf     ADCON0, CHS0        ; select analog channel AN0
        bsf     ADCON0, ADON        ; Turn on ADC module

        bsf     RFENA               ; Enable Transmitter

        ; After selecting a new channel, allow for sufficent sample time.
        ; The amount of sample time depends on the charging time of the
        ; internal charge holding capacitor (Section 7.2).

        movlw   D'6'                ; At 4 MHz, a 22us delay
        movwf   Count               ; (22us = 2us + 6 * 3us + 1us)
        decfsz  Count, F
        goto    $ - 1

        bsf     ADCON0, GO          ; start A/D conversion
        btfsc   ADCON0, GO          ; has A/D conversion completed?
        goto    $ - 1

        bsf     ADCON0, CHS0        ; select analog channel AN1

        movlw   D'6'                ; At 4 MHz, a 22us delay
        movwf   Count               ; (22us = 2us + 6 * 3us + 1us)
        decfsz  Count, F
        goto    $ - 1
        
        rlf		ADRESH, W           ; copy msb to lsb
        movf	ADRESH, W
        movwf	CountHi
        rlf		CountHi, F

;----------------------------------
; Scan push buttons
;----------------------------------

SCAN
        ; Do uninitialized delay before scanning buttons
        ; to reduce overhead and permit faster modulation
        ; when button SW1 is pressed on GP4

        bsf     ADCON0, GO          ; start A/D conversion
        bsf     TXD                 ; Set data pin hi

;----------------------------------------
; 10 bit variable delay with instruction cycle resolution
;
; The bits 7 to 0 of delay time must be in register W, bit 8 is in CountHi
; MSb and bit 9 is in CountHi LSb. All values from 0 to 1023 are acceptable.
; A 15 instruction cycle overhead is always added to the delay time.

        movwf   Count               ; initialize 8 lsb of delay counter
        btfsc   Count, 0            ; if bit 0 set
        goto    $ + 1               ; then do an extra delay cycle
        btfsc   Count, 1            ; if bit 1 set
        goto    $ + 1               ; then do first extra delay cycle
        btfsc   Count, 1            ; if bit 1 set
        goto    $ + 1               ; then do second extra delay cycle
        rlf     CountHi, W          ; non destructive read of bit 8
        rrf     Count, F            ; position counter for loop
        rrf     CountHi, W          ; non destructive read of bit 7
        rrf     Count, F            ; position counter for loop
        incf    Count, F            ; correct for decrement rollover

        movf    ADRESH,W            ; Get the A/D channel 1 reading while waiting
		decfsz  Count,F
         goto   $ - 2               ; loop takes (Delay/4)*4=Delay cycles

        movwf   CountSave           ; Save A/D reading for the delay loop count

        btfsc   PB4                 ; If Push Button GP4 not pressed?
         goto   CHECKGP3            ;  then check GP3

        bcf     TXD                 ; Set data pin low

;----------------------------------------
; 10 bit variable delay with instruction cycle resolution
;
; The bits 7 to 0 of delay time must be in register W, bit 8 is in CountHi
; MSb and bit 9 is in CountHi LSb. All values from 0 to 1023 are acceptable.
; A 15 instruction cycle overhead is always added to the delay time.

		movwf   Count               ; initialize 8 lsb of delay counter
		btfsc   Count, 0            ; if bit 0 set
		 goto   $ + 1               ; then do an extra delay cycle
		btfsc   Count, 1            ; if bit 1 set
		 goto   $ + 1               ; then do first extra delay cycle
		btfsc   Count, 1            ; if bit 1 set
		 goto   $ + 1               ; then do second extra delay cycle
		rlf     CountHi, W          ; non destructive read of bit 8
		rrf     Count, F            ; position counter for loop
		rrf     CountHi, W          ; non destructive read of bit 7
		rrf     Count, F            ; position counter for loop
		incf    Count, F            ; correct for decrement rollover

		movf    CountSave, W        ; preload variable delay length while waiting
		decfsz  Count, F
		 goto   $ - 2               ; loop takes (Delay/4)*4=Delay cycles

        goto    SCAN                ; Loop until button released

CHECKGP3
        btfsc   PB3                 ; If Push Button GP3 not pressed?
         goto   STOP                ;  then stop transmitting
         
        bsf     TXD                 ; Set data pin hi
        goto    SCAN                ; Loop until button released

STOP
        bcf     TXD                 ; Set data pin low
		nop                         ; Delay to prevent back to back bit manipulation
        bcf     RFENA               ; Disable Transmitter, read also clears change latch
        sleep                       ; Put PICmicro to sleep
        goto    MAIN                ; Upon wake-up on pin change, restart


;----------------------------------------------------------------------
; Data EEPROM Memory (Section 8.0)
;
; PIC12F629/675 devices have 128 bytes of data EEPROM with address
; range 0x00 to 0x7F.

        ; Initialize Data EEPROM Memory locations

;        ORG 0x2100
;        DE  0x00, 0x01, 0x02, 0x03


;----------------------------------------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (OSCCAL) (Section 2.2.2.7)
;
; The below statements are placed here so that the program can be
; simulated with MPLAB SIM.  The programmer (PICkit or PROMATE II)
; will save the actual OSCCAL value in the device and restore it.
; The value below WILL NOT be programmed into the device.

        org     0x3ff
        retlw   0x80

;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
