;----------------------------------------------------------------------
;                                                                     
;                     Software License Agreement                      
;                                                                     
; The software supplied herewith by Microchip Technology Incorporated 
; (the "Company") for its PICmicro Microcontroller is intended and   
; supplied to you, the Companys customer, for use solely and         
; exclusively on Microchip PICmicro Microcontroller products.         
;                                                                     
; The software is owned by the Company and/or its supplier, and is     
; protected under applicable copyright laws. All rights are reserved.  
; Any use in violation of the foregoing restrictions may subject the  
; user to criminal sanctions under applicable laws, as well as to     
; civil liability for the breach of the terms and conditions of this  
; license.                                                             
;                                                                      
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,   
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR          
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.                    
;                                                                     
;----------------------------------------------------------------------
;
;   Filename:           rcvr_select.asm
;   Date:               April 25, 2003
;   File Version:       1.0
;   Assembled using:    
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;
;----------------------------------------------------------------------
;
;   Files required:
;                       p16f676.inc
;
;
;----------------------------------------------------------------------
;
;   Program Description 
;
;   This program demonstrates a simple command and control application.
;   When a push button on the transmitter module is depressed, the 
;   corresponding LED is lit on the PICkit(tm) 1 FLASH Starter Kit.
;
;   Pressing transmitter module push button GP3 lights LED D0 on the
;   PICkit(tm) 1. Pressing push button GP4 lights LED D1.
;
;   The protocol is a simplified KeeLoq(r) protocol compatible with
;   the HCS101 fixed code products.  This code was addapted from 
;   Microchip Technology application note AN740.
;
;----------------------------------------------------------------------

    list        p=16f676        ; list directive to define processor
    #include    <p16f676.inc>   ; processor specific variable definitions

;    errorlevel  -302            ; suppress message 302 from list file

; ---------------------------------------------------------------------
; Configuration Bits (Section 9.1 Configuration Bits)
; ---------------------------------------------------------------------
;
; Data Memory Code Protection bit:
; _CPD_ON = Enabled
; _CPD_OFF = Disabled
;
; Program Memory Code protection:
; _CP_ON = Enabled
; _CP_OFF = : Disabled
;
; Brown-out Detection Enable bit:
; _BODEN_ON = Enabled
; _BODEN_OFF = Disabled
;
; GP3/MCLR pin function select:
; _MCLRE_ON = GP3/MCLR pin function is /MCLR
; _MCLRE_OFF = GP3/MCLR pin function is digital I/O, 
;              /MCLR internally tied to Vdd
;
; Power-up Timer Enable bit:
; _PWRTE_ON = Enabled
; _PWRTE_OFF = Disabled
;
; Watchdog Timer Enable bit:
; _WDT_ON = Enabled
; _WDT_OFF = Disabled
;
; Oscillator Selction bits:
; _EXTRC_OSC_NOCLKOUT = CLKOUT function on GP4 pin, RC on GP5 pin.
; _EXTRC_OSC_CLKOUT = I/O function on GP4 pin, RC on GP5 pin.
; _INTRC_OSC_CLKOUT =  Internal oscillator, CLKOUT function on GP4 pin,
;                      I/O function on GP5 pin.
; _INTRC_OSC_NOCLKOUT = Internal oscillator, I/O function on GP4 and GP5 pins.
; _EC_OSC = I/O function on GP4 pin, CLKIN on GP5 pin.
; _HS_OSC = High speed crystal/resonator on GP4 and GP5 pins.
; _XT_OSC = Crystal/resonator on GP4 and GP5 pins.
; _LP_OSC = Low power crystal on GP4 and GP5 pins.
;
;
; ---------------------------------------------------------------------

    __CONFIG    _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT

;----------------------------------------------------------------------
; Variables  (Section 2.2 Data Memory Organization)
;----------------------------------------------------------------------

    ; Data Memory Organization (Section 2.2)
    ;
    ; Register locations 0x20 to 0x5F (64 bytes) are General Purpose 
    ; registers, implemented as static RAM and are mapped across both
    ; banks. 

    cblock  0x20

        w_temp                      ; used for context saving 
        status_temp                 ; used for context saving

        TEMP                        ; General Purpose Temp register
        DATA0			            ; 1st byte of received data
        DATA1			            ; 2nd byte of received data
        DATA2			            ; 3rd byte of received data
        DATA3			            ; 4th byte of received data
        ORIGIN			            ; a reference used to increment TMRLOW
        SX1TMR			            ; LED timer (low order)
        SX2TMR			            ; LED timer (high order)
        TMRLOW			            ; pulse width timer (low order)
        TMRHIGH			            ; pulse width timer (high order)
        HIGHWDTH		            ; high pulse width
        LOWWDTH			            ; low pulse width
        STATECNTR		            ; program state counter
        BITCNTR			            ; data stream bit counter
        FLAGS			            ; flags
        COUNTR                      ; misc. counter

        LEDREG                      ; LED Array Register
        LEDREGBUF                   ; LED Array Register Buffer
        LEDSTATE                    ; LED Array State Counter

    endc

;----------------------------------------------------------------------
; Defines
;----------------------------------------------------------------------

    ;--------------------
    ; PORTA (Section 3.1)
    ;--------------------
    ; PORTA is an 6-bit wide, bi-directional port. The corresponding data
    ; direction register is TRISA. Setting a TRISA bit (= 1) will make
    ; the corresponding PORTA pin and input. Clearing a TRISA bit (= 0)
    ; will make the corresponding PORTA pin an output. The exception is
    ; RA3, which is input only and its TRIS bit will always read as a '1'.
    ;
    ; Function of PORTA pins depend on:
    ;   Configuration Bits (CONFIG) (Section 9.1)
    ;   Weak Pull-up Register (WPU) (Section 3.2.1)
    ;   Interrupt-on-change Register (IOCB) (Section 3.2.2)
    ;   Option Register (OPTION_REG) (Register 4-1)
    ;   TIMER1 Control Register (T1CON) (Register 5-1)
    ;   Comparator Control Register (CMCON) (Section 6.0)
    ;   A/D Control Register (ADCON0) (Section 7.0) (PIC16F676 Only)

#define RA0     PORTA, 0            ; (Analog Input) Potentiometer RP1
#define RA1     PORTA, 1            ; (Digital Input/Output) LEDs D6, D7
#define RA2     PORTA, 2            ; (Digital Input/Output) LEDs D2, D3, D4, D5, D6, D7
#define LRNPB   PORTA, 3            ; (Digital Input Only) Push Button SW1
#define RA4     PORTA, 4            ; (Digital Input/Output) LEDs D0, D1, D2, D3
#define RA5     PORTA, 5            ; (Digital Input/Output) LEDs D0, D1, D4, D5

    ; Define for TRISA Register (Section 3.1)

    ;   PORTA Pins = xx543210
#define PORTATRIS  b'00111111'

    ;--------------------
    ; PORTC (Section 3.3)
    ; --------------------
    ; PORTC is a general purpose I/O port consisting of 6 bi-directional
    ; pins. The pins can be configured for either digital I/O or analog
    ; input to A/D converter. For specific information about individual
    ; functions such as the comparator or the A/D.

#define RC0     PORTC, 0            ; (Digital Input/Output)
#define RC1     PORTC, 1            ; (Digital Input/Output)
#define RC2     PORTC, 2            ; (Digital Input/Output)
#define RC3     PORTC, 3            ; (Digital Input/Output)
#define RC4     PORTC, 4            ; (Digital Input/Output)
#define RXDATA  PORTC, 5            ; (Digital Input) Receive Data

    ;   PORTC Pins = xx543210
#define PORTCTRIS  b'00111111'

;----------------------------------------
; Program Defines
;----------------------------------------

    ; Status Counter

#define	BEGN	0x00
#define	BEGN1	0x01
#define	HEADR	0x02
#define	HEADR1	0x03
#define	HIGHP	0x04
#define	LOWP	0x05
#define	RECRD	0x06
#define	WAIT	0x07
#define	VALID	0x08
#define	IMPLMNT	0x09

    ; Flags

#define LRNFLG  FLAGS, 0            ; this flag is set when in learn mode
#define TOGFLG  FLAGS, 1            ; toggle flag
#define HLFLG   FLAGS, 2            ; high-low flag

    ; LEDs

#define LED0    LEDREG, 0           ; S0 LED
#define LED1    LEDREG, 1           ; S1 LED
#define LED2    LEDREG, 2           ; S2 LED
#define LED3    LEDREG, 3
#define LED4    LEDREG, 4
#define LED5    LEDREG, 5
#define LED6    LEDREG, 6
#define LED7    LEDREG, 7

#define LRNLED  LED3

    ;     GPIO Pins = xx543210
#define LED0TRIS    b'00001111'
#define LED1TRIS    b'00001111'
#define LED2TRIS    b'00101011'
#define LED3TRIS    b'00101011'
#define LED4TRIS    b'00011011'
#define LED5TRIS    b'00011011'
#define LED6TRIS    b'00111001'
#define LED7TRIS    b'00111001'
#define LEDOFFTRIS  b'00111111'

    ;     GPIO Pins = xx543210
#define LED0ON      b'00010000'
#define LED1ON      b'00100000'
#define LED2ON      b'00010000'
#define LED3ON      b'00000100'
#define LED4ON      b'00100000'
#define LED5ON      b'00000100'
#define LED6ON      b'00000100'
#define LED7ON      b'00000010'


;----------------------------------------------------------------------
; Program Memory
;----------------------------------------------------------------------

    ; Program Memory Organization (Section 2.1)

        ORG     0x000               ; RESET Vector

        nop                         ; for ICD use
        goto    RESET


        ORG     0x004               ; Interrupt Vector

        movwf   w_temp              ; save W register
        swapf   STATUS, W           ; swap status to be saved into W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movwf   status_temp         ; save STATUS register

;----------------------------------------
; Interrupt Service Routine (ISR) (Section 9.4)
;
; Description: This program does not use interrupts
;
;----------------------------------------

        swapf   status_temp, W      ; swap status_temp into W, sets bank to original state
        movwf   STATUS              ; restore STATUS register
        swapf   w_temp, F
        swapf   w_temp, W           ; restore W register

        retfie


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_READ
;
; Description: To read an EEPROM data memory location, the address is
;   written to the EEADR register and set control bit RD (EECON1<0>) to
;   initiate a read. Data is available in the EEDATA register the next
;   clock cycle.
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: W contains EEPROM address to be read
;   
; Output: W contains EEPROM data
;
;----------------------------------------------------------------------

DATA_EEPROM_READ

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movwf   EEADR               ; move EEPROM address in W to EEADR
        bsf     EECON1, RD          ; initiate EEPROM read
        movf    EEDATA, W           ; move data to W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_WRITE
;
; Description: To write an EEPROM data memory location, the address is
;   written to the EEADR register, data to the EEDATA register, then
;   execute a required sequence of instructions.
;   
; CAUTION: Interrupts are disable during this subroutine
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: Address = EEADR, Data = EEDATA
;   
; Output: none
;
;----------------------------------------------------------------------

DATA_EEPROM_WRITE

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bsf     EECON1, WREN        ; EEPROM Write Enable: allow write cycles
        bcf     INTCON, GIE         ; disable global interrupts
                                    ; *** required sequence, do not alter ***
        movlw   0x55
        movwf   EECON2
        movlw   0xAA
        movwf   EECON2
        bsf     EECON1, WR          ; initiate EEPROM write
                                    ; *** end required sequence ***

        btfsc   EECON1, WR          ; has write completed?
        goto    $-1

;        bsf     INTCON, GIE         ; enable global interrupts
        bcf     EECON1, WREN        ; EEPROM Write Enable: inhibit write cycles

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return


;----------------------------------------------------------------------
; Subroutine READ_ANALOG_AN0
;
; Description: Read analog channel 0 (AN0).
;   
; Constants: none
;   
; Global Variables: none
;   
; Initialization: none
;   
; Output: ADRESH and ADRESL contain 10-bit A/D result justified 
;   according to ADCON0, ADFM bit.
;
;----------------------------------------------------------------------

READ_ANALOG_AN0

        bsf     ADCON0, ADON        ; Turn on ADC module

        bcf     ADCON0, CHS1        ; select analog channel AN0
        bcf     ADCON0, CHS0

        ; After selecting a new channel, allow for sufficent sample time.
        ; The amount of sample time depends on the charging time of the
        ; internal charge holding capacitor (Section 7.2).

        movlw   0x07                ; At 4 MHz, a 22 us delay
        movwf   TEMP                ; (22us = 2us + 6 * 3us + 1us)
        decfsz  TEMP, F
        goto    $-1

        bsf     ADCON0, GO          ; start A/D conversion

        btfsc   ADCON0, GO          ; has A/D conversion completed?
        goto    $-1

        bcf     ADCON0, ADON        ; Turn off ADC module (consumes no operating current)

        return


;----------------------------------------------------------------------
; Initialize PICmicro (PIC12F675)
;----------------------------------------------------------------------

INITIAL

; Disable global interrupts during initialization

        bcf     INTCON, GIE         ; disable global interrupts


;----------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (OSCCAL) (Section 2.2.2.7)
;
; A calibration instruction is programmed into the last location of
; program memory. This instruction is a RETLW XX, where the literal is
; the calibration value. The literal is placed in the OSCCAL register
; to set the calibration of the internal oscillator.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        call    0x3FF               ; retrieve factory calibration value
        movwf   OSCCAL              ; update register with factory cal value

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; PORTS A AND C (Section 3.0)
;
; Store GPTRIS value defined above into the TRISIO direction register

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movlw   PORTATRIS
        movwf   TRISA               ; Write to TRISA register

        movlw   PORTCTRIS
        movwf   TRISC               ; Write to TRISC register

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; Comparator Module (Section 6.0)
;
; The PIC16F630/676 devices have one analog comparator. The inputs to
; the comparator are multiplexed with the RA0 and RA1 pins. There is
; an on-chip Comparator Voltage Reference that can also be applied to
; an input of the comparator. In addition, RA2 can be configured as
; the comparator output. The Comparator Control Register (CMCON)
; contains bits to control the comparator. The Voltage Reference
; Control Register (VRCON) controls the voltage reference module.

        ; Comparator Configuration (Figure 6-2)
;        bcf     CMCON, CINV         ; Comparator Output Inversion: not inverted
;        bcf     CMCON, COUT         ; Comparator Output bit: Vin+ < Vin-
;        bcf     CMCON, CIS          ; Comparator Input Switch: Vin- connectos to Cin-

        ; CM2:CM0 = 111 - Comparator Off (lowest power)
        bsf     CMCON, CM2          ; Comparator Mode bit 2
        bsf     CMCON, CM1          ; Comparator Mode bit 1
        bsf     CMCON, CM0          ; Comparator Mode bit 0

        ; VRCON (Register 6-2)
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     VRCON, VREN         ; CVref circuit: powered down, no Idd drain

;        bcf     VRCON, VRR          ; CVref Range Selection: High Range

;        bcf     VRCON, VR3          ; CVref value selection bit 3
;        bcf     VRCON, VR2          ; CVref value selection bit 2
;        bcf     VRCON, VR1          ; CVref value selection bit 1
;        bcf     VRCON, VR0          ; CVref value selection bit 0

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; Analog-to-Digital Converter (A/D) Module (Section 7.0) (PIC16F676 Only)
;
; The analog-to-digital converter (A/D) allows conversion of an analog
; input signal to a 10-bit binary representation of that signal. The
; PIC16F676 has eight analog inputs multiplexed into one sample and hold
; circuit. There are two registers to control the functions of the A/D
; module:
;   A/D Control Register 0 (ADCON0)
;   A/D Control Register 1 (ADCON1)
;   Analog Select Register (ANSEL)
;
; Note: When using PORTA or PORTC pins as analog inputs, ensure the 
;       TRISA or TRISC register bits are set (= 1) for input.

        bcf     ADCON0, ADFM        ; A/D Result Formed: left justified
        bcf     ADCON0, VCFG        ; Voltage Reference: Vdd

        bcf     ADCON0, ADON        ; ADC is shut-off and consumes no operating current

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        ; select A/D Conversion Clock Source: Fosc/8
        bcf     ADCON1, ADCS2       ; A/D Conversion Clock Select bit 2
        bcf     ADCON1, ADCS1       ; A/D Conversion Clock Select bit 1
        bsf     ADCON1, ADCS0       ; A/D Conversion Clock Select bit 0

        ; select GPIO pins that will be analog inputs: RA0/AN0
        bcf     ANSEL, ANS7         ; Analog Select RC3/AN7: digital I/O
        bcf     ANSEL, ANS6         ; Analog Select RC2/AN6: digital I/O
        bcf     ANSEL, ANS5         ; Analog Select RC1/AN5: digital I/O
        bcf     ANSEL, ANS4         ; Analog Select RC0/AN4: digital I/O
        bcf     ANSEL, ANS3         ; Analog Select RA3/AN3: digital I/O
        bcf     ANSEL, ANS2         ; Analog Select RA2/AN2: digital I/O
        bcf     ANSEL, ANS1         ; Analog Select RA1/AN1/Vref: digital I/O
        bsf     ANSEL, ANS0         ; Analog Select RA0/AN0: analog input

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; TIMER1 Module with Gate Control (Section 5.0)
;
; The TIMER1 Control Register (T1CON) is used to enable/disable TIMER1
; and select various features of the TIMER1 module.

        bcf     T1CON, TMR1ON       ; TIMER1: stopped

        bcf     T1CON, TMR1CS       ; TIMER1 Clock Source Select: Internal Clock (Fosc/4)

        bcf     T1CON, NOT_T1SYNC   ; TIMER1 External Clock Input Sync Control: Syncronize external clock input

        ; T1OSCEN only if INTOSC without CLKOUT oscillator is active, else ignored
        bcf     T1CON, T1OSCEN      ; LP Oscillator Enable Control: LP oscillator off

        ; TIMER1 Input Prescale Select: 1:1
        bcf     T1CON, T1CKPS1      ; TIMER1 Input Clock Prescale Select bit 1
        bcf     T1CON, T1CKPS0      ; TIMER1 Input Clock Prescale Select bit 0

        ; TMR1GE only if TMR1ON = 1, else ignored
        bcf     T1CON, TMR1GE       ; TIMER1 Gate Enable: on


;----------------------------------------
; PORTA Weak Pull-up Register (WPUA) (Section 3.2.1)
;
; Each of the PORTA pins, except RA3, has an individually configurable
; weak internal pull-up. Control bits WPUAx enable or disable each 
; pull-up. Refer to Register 3-1. Each weak pull-up is automatically
; turned off when the port pin is configured as an output. The pull-ups
; are disabled on a Power-on Reset by the /RAPU bit (see OPTION Register
; below).

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

;    PORTA Pins = xx54x210
        movlw   B'00000000'         ; no pull-ups enabled
        movwf   WPUA

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; OPTION Register (OPTION_REG) (Section 2.2.2.2)
; TIMER0 Module (Section 4.0)
;
; The OPTION_REG contains control bits to configure:
;   Weak pull-ups on GPIO (see also WPU Register above)
;   External RA2/INT interrupt
;   TMR0
;   TMR0/WDT prescaler

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bsf     OPTION_REG, NOT_GPPU ; GPIO pull-ups: disabled

        bsf     OPTION_REG, INTEDG  ; Interrupt Edge: on rising edge of RA2/INT pin

        bcf     OPTION_REG, T0CS    ; TMR0 Clock Source: internal instruction cycle (CLKOUT)
        bcf     OPTION_REG, T0SE    ; TMR0 Source Edge: increment low-to-high transition on GP2/T0CKI pin

        bcf     OPTION_REG, PSA     ; Prescaler Assignment: assigned to TIMER0

        ; TMR0 Prescaler Rate: 1:4
        bcf     OPTION_REG, PS2     ; Prescaler Rate Select bit 2
        bcf     OPTION_REG, PS1     ; Prescaler Rate Select bit 1
        bsf     OPTION_REG, PS0     ; Prescaler Rate Select bit 0

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; PORTA Interrupt-on-Change Register (IOCA) (Section 3.2.2)
;
; Each of the PORTA pins is individually configurable as an interrupt-
; on-change pin. Control bits IOCAx enable or disable the interrupt 
; function for each pin. Refer to Register 3-4. The interrupt-on-change
; is disabled on a Power-on Reset.
;
; Note: Global interrupt enables (GIE and GPIE) must be enabled for
;       individual interrupts to be recognized.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        clrf    IOCA                ; Interrupt-on-change disabled

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; Peripheral Interrupt Enable Register (PIE1) (Section 2.2.2.4)
;
; The PIE1 register contains peripheral interrupt enable bits.
;
; Note: The PEIE bit (INTCON<6>) must be set to enable any 
;       peripheral interrupt.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     PIE1, EEIE          ; EE Write Complete Interrupt: disabled
        bcf     PIE1, ADIE          ; A/D Converter Interrupt (PIC12F675 Only): disabled
        bcf     PIE1, CMIE          ; Comparator Interrupt: disabled
        bcf     PIE1, TMR1IE        ; TMR1 Overflow Interrupt: disabled

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; Interrupt Control Register (INTCON) (Section 2.2.2.3)
;
; The INTCON register contains enable and disable flag bits for TMR0
; register overflow, GPIO port change and external GP2/INT pin 
; interrupts.

        bcf     INTCON, PEIE        ; disable Peripheral Interrupt Enable bit
        bcf     INTCON, T0IE        ; disable TMR0 Overflow Interrupt Enable bit
        bcf     INTCON, INTE        ; disable GP2/INT External Interrupt Enable bit
        bcf     INTCON, RAIE        ; disable Port Change Interrupt Enable bit

        bcf     INTCON, GIE         ; disable global interrupts

        retlw   0                   ; exit INITIAL 


;----------------------------------------------------------------------
; Subroutine: TIMER
;   
; Description: Continually updates two higher order timers (SX1TMR and
;   SX2TMR) for use in LED timing.
;   
; Constants: none
;   
; Global Variables: SX1TMR, SX2TMR
;   
; Initialization: none
;   
; Output: SX1TMR, SX2TMR
;   
;----------------------------------------------------------------------

TIMER
        btfss   TOGFLG              ; TOGGLE forces this routine to spend
         goto   TIMER1              ;   1/2 of TMR0 in TIMER and 1/2 in TIMER1.

        movlw   B'01111111'
        addwf   TMR0, W             ; TOGGLE toggles back and forth to a   
        btfss   STATUS, C           ;    one the rate TMR0 overflows.
         retlw  0                   ; TMR0 overflow ~= 1ms (2^8*4us)

        bcf     TOGFLG
        incfsz  SX1TMR, F           ; SX1TMR resolution ~= 1ms
         retlw  0                   ; SX1TMR overflow ~= 0.25sec (2^8*1ms) 

        incf    SX2TMR, F           ; SX2TMR resolution ~= 0.25sec
        retlw   0                   ; SX2TMR overflw ~= 1min (2^8*0.23sec)  

TIMER1
        movlw   B'01111111'         ; Timer routine spends half its time
        addwf   TMR0, W             ;   in TIMER1 waiting to set TOGGLE
        btfsc   STATUS, C           ;   to one again
         retlw  0

        bsf     TOGFLG
        retlw   0


;----------------------------------------------------------------------
; Subroutine: CLOCK
;   
; Description: Continually updates TMRLOW and TMRHIGH.
;   
; Constants: none
;   
; Global Variables: ORIGIN
;   
; Initialization: ORIGIN
;   
; Output: TMRLOW, TMRHIGH
;   
;----------------------------------------------------------------------

CLOCK
        movf    ORIGIN, W           ; TMRLOW is updated based on time passed since ORIGIN was set.
        subwf   TMR0, W             ; TMRLOW resolution ~= 4us (like TMR0)  
        addwf   TMRLOW, F           ; TMRLOW overflow ~= 1ms (2^8*4ms)
        btfsc   STATUS, C           ; TMRHIGH resolution ~= 1ms
         incf   TMRHIGH, F          ; TMRHIGH overflow ~= 0.24sec (2^8*1ms)

        nop                         ; NOP and subtraction commands ensure 
        nop                         ;   ORIGIN equals TMR0 as called upon
        nop                         ;   in line 2 of CLOCK.
        movlw   2
        subwf   TMR0, W 
        movwf   ORIGIN              ; (ORIGIN must be updated to equal the value
                                    ;   of TMR0 at time of operation with ORIGIN.)
        retlw   0

;----------------------------------------------------------------------
; Subroutine: LRNDTCT
;   
; Description: When the LEARN pushbutton is pressed, this function places
;   the program in LEARN mode by:
;   - setting the LERN flag
;   - setting the STATCNTR to BEGN
;   - turning on the learn LED
;   - clearing the higher order timers, SX1TMR and SX2TMR
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: none
;   
; Output: LRNLED, LERN
;   
;----------------------------------------------------------------------

LRNDTCT
        btfsc   LRNFLG              ; if not already in Learn Mode
         retlw  0

        movlw   BEGN                ; set STATCNTR to BEGN
        movwf   STATECNTR

        bsf     LRNFLG              ; set LERN flag

        bsf     LRNLED              ; turn on the Learn LED

        clrf    SX1TMR              ; clear SX Timer Counter
        clrf    SX2TMR

        retlw   0


;----------------------------------------------------------------------
; Subroutine: LEARN
;   
; Description: This routine learns the first two bytes of data received
;   from the transmitter by storing these bytes in its internal EEPROM.
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: none
;   
; Output: none
;   
;----------------------------------------------------------------------

LEARN	
        btfss   SX2TMR, 5           ; If valid reception is completed within 8 seconds (2^5*0.25sec)
         goto   LEARN1              ;   goto LEARN1

        bcf     LRNFLG              ;   else exit LEARN mode  
        bcf     LRNLED

        retlw   0

LEARN1
        movlw   VALID               ; If the State Counter currently holds  
        xorwf   STATECNTR, W        ;   the value for exectuting the 
        btfss   STATUS, Z           ;   VALIDATE function, then a success-  
         retlw  0                   ;   ful reception has occurred. 

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movlw   0x00                ; EEPROM Address = 0x00
        movwf   EEADR

        movfw   DATA0               ; EEPROM Data = DATA0
        movwf   EEDATA

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        call    DATA_EEPROM_WRITE   ; write first byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movlw   0x01                ; EEPROM Address = 0x01
        movwf   EEADR

        movfw   DATA1               ; EEPROM Data = DATA1
        movwf   EEDATA

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        call    DATA_EEPROM_WRITE   ; write second byte to EEPROM
	  
        bcf     LRNLED              ; exit learn mode
        bcf     LRNFLG

        movlw   BEGN                ; set STATECNTR to BEGN
        movwf   STATECNTR

        retlw   0                   ; exit subroutine LEARN


;----------------------------------------------------------------------
; Subroutine: SETWATCH
;   
; Description: Initialize the pulse width timer registers.	
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: none
;   
; Output: ORIGIN
;   
;----------------------------------------------------------------------

SETWATCH
        movf    TMR0, W             ; record TMR0's value in ORIGIN 
        movwf   ORIGIN
        clrf    TMRLOW              ; clear the low and high order timers
        clrf    TMRHIGH

        retlw	0


;----------------------------------------------------------------------
; Subroutine: SXON
;   
; Description: Turns off all LEDs when they timeout.
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: SX2TMR
;   
; Output: 
;   
;----------------------------------------------------------------------

SXON
        btfss   SX2TMR, 0           ; When SX1TMR overflows, SX2TMR will increment to 1.  Recall this
         retlw   0                  ;    will occur at 0.25 seconds (2^8*1ms) after SX1TMR is initiated.

        bcf     LED0
        bcf     LED1
        bcf     LED2

        retlw   0


;----------------------------------------------------------------------
; Subroutine: LEDSTATEMACHINE
;   
; Description: This is the state machine that cycles through the LED's
;   Atates are incremented in the interrupt routine after an overflow on TMR0
;
; Constants: 
;   
; Global Variables: 
;   
; Initialization: 
;   
; Output: 
;   
;----------------------------------------------------------------------

LEDStateMachine
        movlw   HIGH LEDStateMachineStart
        movwf   PCLATH
        movf    LEDSTATE, W         ; Mask bits (should be only 8 states)
        andlw   B'00000111'

        addwf   PCL, F

LEDStateMachineStart
        goto    LEDSTATE0
        goto    LEDSTATE1
        goto    LEDSTATE2
        goto    LEDSTATE3
        goto    LEDSTATE4
        goto    LEDSTATE5
        goto    LEDSTATE6
        goto    LEDSTATE7

LEDSTATE0
        call    LITELED0
        goto    EndLEDStateMachine

LEDSTATE1
        call    LITELED1
        goto    EndLEDStateMachine

LEDSTATE2
        call    LITELED2
        goto    EndLEDStateMachine

LEDSTATE3
        call    LITELED3
        goto    EndLEDStateMachine

LEDSTATE4
        call    LITELED4
        goto    EndLEDStateMachine

LEDSTATE5
        call    LITELED5
        goto    EndLEDStateMachine

LEDSTATE6
        call    LITELED6
        goto    EndLEDStateMachine

LEDSTATE7
        call    LITELED7
	
EndLEDStateMachine
        return


;----------------------------------------
	
LITELED0
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   LED0TRIS
        movwf   TRISA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movlw   LED0ON
        movwf   PORTA
        return
	
LITELED1
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   LED1TRIS
        movwf   TRISA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movlw   LED1ON
        movwf   PORTA
        return

LITELED2
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   LED2TRIS
        movwf   TRISA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movlw   LED2ON
        movwf   PORTA
        return
	
LITELED3
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   LED3TRIS
        movwf   TRISA	
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movlw   LED3ON
        movwf   PORTA
        return
	
LITELED4
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   LED4TRIS
        movwf   TRISA				
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movlw   LED4ON
        movwf   PORTA
        return

LITELED5
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   LED5TRIS
        movwf   TRISA			
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movlw   LED5ON
        movwf   PORTA
        return

LITELED6
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   LED6TRIS
        movwf   TRISA		
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movlw   LED6ON
        movwf   PORTA
        return

LITELED7
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   LED7TRIS
        movwf   TRISA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movlw   LED7ON
        movwf   PORTA
        return


;----------------------------------------------------------------------
; Subroutine: Display
;   
; Description: Displays Value Stored In LEDREGBUF On LED Array
;   1 bit is displayed during each call
;   LEDREGBUF is reloaded with LEDREG after 8 calls of DISPLAY()          
;   D7..D4 LED'S show most significant nibble
;   D3..D0  LED'S show least significant nibble
;
; Constants: 
;   
; Global Variables: LEDREG, LEDREGBUF
;   
; Initialization: 
;   
; Output: 
;   
;----------------------------------------------------------------------

DISPLAY
        movf    LEDSTATE, W
        btfsc   STATUS, Z           ; Are we at state 0?	  
         goto   BUFFERRESET         ; Yes
        btfss   LEDSTATE, 3         ; No, did we pass the last state?
         goto   BUFFERLOADED        ;   No

BUFFERRESET                         ;   Yes
        movf    LEDREG, W
        btfss   STATUS, Z           ; is LEDREG = 0?
         goto   BUFFERRESET1        ;   no, jump over
                                    ;   yes, turn off LEDs
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   LEDOFFTRIS
        movwf   TRISA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        clrf    PORTA

BUFFERRESET1
        movf    LEDREG, W
        movwf   LEDREGBUF           ; Reload with the current value in LEDREGISTER
        clrf    LEDSTATE            ; Ensure State Zero
	
BUFFERLOADED
        bcf     STATUS, C           ; Ensure Carry Is Clear
        rrf     LEDREGBUF, F
        btfsc   STATUS, C           ; Is The Next Bit a '1'
         goto   UPDATELED           ;   Yes, Update The LED Array
        goto    CONTINUE            ;   No

UPDATELED
        call    LEDStateMachine     ; Update LED Array
		
CONTINUE
        incf    LEDSTATE, F         ; Point To Next State
        return


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Main Program
;----------------------------------------------------------------------
;----------------------------------------------------------------------

RESET
        clrf    FLAGS               ; clear flags
        clrf    PORTA               ; initialize PORTA inputs and outputs
        clrf    PORTC               ; initialize PORTC inputs and outputs
        clrf    LEDREG
        clrf    LEDSTATE
        clrf    LEDREGBUF
        movlw   BEGN                ; setup the state counter to call BEGIN 
        movwf   STATECNTR
        call    INITIAL

MAIN
        call    TIMER
        call    CLOCK
        call    DISPLAY             ; Update LED Array (light LEDs)

        movlw   B'00000111'         ; check if LEDs S0, S1, or S2 are set
        andwf   LEDREG, W           ; if set call SXOB
        btfss   STATUS, Z
         call   SXON	

        btfss   LRNPB               ; if learn button is pushed call 
         call   LRNDTCT             ;   LRNDTCT

        btfsc   LRNFLG              ; if in learn mode call LEARN
         call   LEARN

        movlw   HIGH STATEM
        movwf   PCLATH
        movf    STATECNTR, W        ; Mask out the high order bits of 
        andlw   B'00001111'         ;   STATECNTR (a noise guard)

        addwf   PCL, F              ; The program clock (PCL) is incre-
STATEM
        goto    BEGIN               ;   mented by STATECNTR in order
        goto    BEGIN1              ;   to go to the appropiate routine
        goto    HEADER	
        goto    HEADER1
        goto    HIGHPLSE
        goto    LOWPULSE
        goto    RECORD
        goto    WAIT4END
        goto    VALIDATE
        goto    IMPLEMNT
        goto    RESET               ; These RESET commands correct
        goto    RESET               ;   erroneous values of STATECNTR   
        goto    RESET               ;   not caught by the mask above.    
        goto    RESET
        goto    RESET
        goto    RESET


;----------------------------------------------------------------------
; RECEIVE SEQUENCE
;----------------------------------------------------------------------

;*********************** CODE WORD FORMAT ****************************
;    _   _   _   _               _    __   _        __   _           _
;   | | | | | | | |             | |  |  | | |      |  | | |         | 
;   | |_| |_| |_| |_____________| |__|  |_| |_/  /_|  |_| |_________|  
;                                            /  /   
;   |--PREAMBLE---|---HEADER----|---------DATA------------|--GUARD--|
;                                                            TIME
;*********************************************************************

;******************** RECEIVED DATA BREAKDOWN ************************
;
;bytes |        DATA1          |        DATA0          |
;bits  | 7| 6| 5| 4| 3| 2| 1| 0| 7| 6| 5| 4| 3| 2| 1| 0| 
;desc. |S2|S1|S0|S3| 0| 0|      SERIAL NUMBER 3        | 
;
;bytes |        DATA3          |        DATA2          |
;bits  | 7| 6| 5| 4| 3| 2| 1| 0| 7| 6| 5| 4| 3| 2| 1| 0| 
;desc. |                    COUNTER                    |
;
;* S3 Not used
;
;*********************************************************************

;----------------------------------------
; BEGIN
;	This function looks for a possible start to the data stream.	
;	
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			none

BEGIN
        btfsc   RXDATA
         incf   STATECNTR, F        ; Make state BEGIN1
        goto    MAIN

BEGIN1
        btfsc   RXDATA
         goto   MAIN

        call    SETWATCH
        incf    STATECNTR, F        ; Make state HEADER

        goto    MAIN


;----------------------------------------
; HEADER
;	Detects a valid header.	
;	
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			none

HEADER	
        btfsc   RXDATA              ; The program loops here until 1.25ms
         goto   RESTART             ;   passes and if the data is still

        btfss   TMRHIGH, 0          ;   low.  If both hold true -> HEADER1.
         goto   MAIN                ; 1.25ms occurs when:

        movlw   D'64'               ;   TMRHIGH = 1 ~= 2^8*4us = 1ms
        andwf   TMRLOW, W           ;   TMRLOW = 64 ~= 64*4us = 0.25ms
        btfsc   STATUS, Z
         goto   MAIN

        incf    STATECNTR, F        ; Make state HEADER1

        goto    MAIN

HEADER1
        movlw   D'6'                ; If the data goes high before 6ms 
        subwf   TMRHIGH, W          ;   then the header is valid, else
        btfss   STATUS, C           ;   restart.	
         goto    HEADER2            ; TMRHIGH = 6 = 6*1ms = 6ms	

        goto    RESTART

HEADER2
        btfss   RXDATA
         goto   MAIN

        call    SETWATCH		
        movlw   D'32'               ; Initiate BITCNTR to 32 in order to
        movwf   BITCNTR             ;   receive 32 bits of the data stream.
        incf    STATECNTR, F        ; Make state HIGHPLSE

        goto    MAIN


;----------------------------------------
; HIGHPLSE
;	Times the width of high pulses.	
;	
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			none


HIGHPLSE
        btfsc   TMRHIGH, 0          ; If TMRLOW overflows then RESTART
         goto   RESTART

        btfsc   RXDATA		
         goto   MAIN

        movf    TMRLOW, W           ; Move the pulse width value to 
        movwf   HIGHWDTH            ;   HIGHWDTH for later calculations.
        call    SETWATCH
        incf    STATECNTR, F        ; Make state LOWPULSE

        goto    MAIN


;----------------------------------------
;  LOWPULSE
;	Times the width of low pulses.	
;	
;		Input Variables:  
;			none
;		Output Variables: 
;			none


LOWPULSE
        btfsc   TMRHIGH, 0          ; If TMRLOW overflows then make
         goto   LOW2                ;   state HEADER.

        btfss   RXDATA
         goto   MAIN

        movf    TMRLOW, W           ; Move the pulse width value to 
        movwf   LOWWDTH             ;   LOWWDTH for later calculations.
        call    SETWATCH 
        incf    STATECNTR, F        ; Make state RECORD

        goto    MAIN

LOW2
        movlw   HEADR
        movwf   STATECNTR           ; Make state HEADER if lowpulse is
                                    ;   too long.
        goto    MAIN


;----------------------------------------
;  RECORD
;	Records each bit as it comes in from the data stream.	
;	
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			DATA0
;			DATA1
;			DATA2
;			DATA3

RECORD	
        movf    HIGHWDTH, W
        subwf   LOWWDTH, W          ; The state of the carry bit after 
        rrf     DATA3, F            ;   this operation reflects the data
        rrf     DATA2, F            ;   logic.  This is then rotated
        rrf     DATA1, F            ;   into the storage bytes.
        rrf     DATA0, F
        movlw   HIGHP
        movwf   STATECNTR
        decfsz  BITCNTR, F
         goto   MAIN

        movlw   D'4'                ; Starting here and including RECORD1
        movwf   COUNTR              ;   a check is made to make sure that

        movlw   DATA0               ;   the data is not composed entirely
        movwf   FSR                 ;   of 1s.

RECORD1
        movlw   0xFF		
        xorwf   INDF, W             ; Use indirect referencing  to point to
        btfss   STATUS, Z           ;   DATA0 -- DATA3 on subsequent loops 
         goto   RECORD2             ;   in RECORD1.

        incf    FSR, F
        decfsz  COUNTR, F
         goto   RECORD1

        goto    RESTART

RECORD2
        movlw   WAIT		
        movwf   STATECNTR           ; Make state WAIT4END

        goto    MAIN


;----------------------------------------
;  WAIT4END
;	Wait for the guard time at the end of the code word before	
;	attempting to receive another code word.
;
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			none


WAIT4END
        btfsc   HLFLG               ; HIGHLOW is set to indicate that the 
         goto   WAIT1               ;   data has transitioned from a high

        btfsc   RXDATA              ;   to a low.  
         goto   MAIN

        call    SETWATCH
        bsf     HLFLG
WAIT1
        btfss   RXDATA
         goto   WAIT2

        bcf     HLFLG

        goto    MAIN

WAIT2
        btfss   TMRHIGH, 3          ; If the low period is greater than
         goto   MAIN                ;   8ms (2^3*1ms) then the guard time 

        bcf     HLFLG               ;   has been reached.
        incf    STATECNTR, F        ; Make state VALIDATE

        goto    MAIN


;----------------------------------------
;  VALIDATE
;	Checks that the transmission received is from the valid
;	transmitter.
;
;		Input Variables:  
;			DATA0
;			DATA1
;		Output Variables: 
;			none


VALIDATE
        movlw   0x00                ; EEPROM Address = 0x00
        call    DATA_EEPROM_READ    ; Get stored DATA0, EEDATA -> W

        xorwf   DATA0, W            ; compare with received DATA0
        btfss   STATUS, Z           ; If first byte checks out then 
         goto   RESTART             ;   continue, else restart

        movlw   0x01                ; EEPROM Address = 0x01
        call    DATA_EEPROM_READ    ; Get stored DATA1, EEDATA -> W

        movwf   TEMP                ; move received DATA1 -> TEMP
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movf    EEDATA, W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        xorwf   TEMP, F             ; compare with received DATA1

        btfsc   TEMP, 0             ; Do bits 8 and 9 of Serial Number 3
         goto   RESTART             ;   check out?

        btfsc   TEMP, 1
         goto   RESTART

        incf    STATECNTR, F        ; Make state IMPLEMENT

        goto    MAIN


;----------------------------------------
;  IMPLEMNT
;	Implements the outputs specified by the received code word.
;	
;		Input Variables:  
;			DATA1
;		Output Variables: 
;			S0
;			S1
;			S2


IMPLEMNT
        btfsc   DATA1, 7            ; set outputs in accordance with code
        bsf     LED2                ;   word

        btfss   DATA1, 7	
        bcf     LED2

        btfsc   DATA1, 6
        bsf     LED1

        btfss   DATA1, 6
        bcf     LED1

        btfsc   DATA1, 5
        bsf     LED0

        btfss   DATA1, 5
        bcf     LED0

;        movfw   DATA3               ; read received analog value
;        andlw   B'11111100'         ; mask top 6 bits
;        movwf   TEMP                ; save to TEMP
;        bcf     STATUS, C           ; shift right twice
;        rrf     TEMP, F
;        rrf     TEMP, W
;        movwf   LEDREG              ; save to LEDREG

        clrf    SX1TMR              ; initialize the timers for the 
        clrf    SX2TMR              ;   outputs

        goto    RESTART	


;----------------------------------------
;  RESTART
;	Sets the State Counter to BEGIN so that the receive sequence 
;	is restarted.
;
;		Input Variables:  
;			none
;		Output Variables: 
;			none


RESTART
        movlw   BEGN                ; restart receive sequence and return
        movwf   STATECNTR           ;   to MAIN

        goto    MAIN


;----------------------------------------------------------------------
; Data EEPROM Memory (Section 8.0)
;
; PIC12F629/675 devices have 128 bytes of data EEPROM with address
; range 0x00 to 0x7F.

        ; Initialize Data EEPROM Memory locations

        ORG     0x2100
        DE      0x73, 0x43, 0x02, 0x03

;----------------------------------------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (OSCCAL) (Section 2.2.2.7)
;
; The below statements are placed here so that the program can be
; simulated with MPLAB SIM or emulated with the ICD2 or ICE-2000.  
;
; The programmer (PICkit or PROMATE II) will save the actual OSCCAL 
; value in the device and restore it. The value below WILL NOT be
; programmed into the device.

        org     0x3ff
        retlw    0x40

;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
